@@ -12,6 +12,8 @@ from werkzeug.datastructures import FileStorage
 
 from ul_api_utils.access import GLOBAL_PERMISSION__PUBLIC, PermissionDefinition, GLOBAL_PERMISSION__PRIVATE_RT, GLOBAL_PERMISSION__PRIVATE
 from ul_api_utils.api_resource.api_resource_config import ApiResourceConfig
+from ul_api_utils.api_resource.api_resource_error_handling import WEB_EXCEPTION_HANDLING_PARAMS__MAP, \
+    ProcessingExceptionsParams, WEB_UNKNOWN_ERROR_PARAMS
 from ul_api_utils.api_resource.api_resource_fn_typing import ApiResourceFnTyping
 from ul_api_utils.api_resource.api_resource_type import ApiResourceType
 from ul_api_utils.api_resource.api_response import JsonApiResponse, HtmlApiResponse, FileApiResponse, JsonApiResponsePayload, EmptyJsonApiResponse, \
@@ -20,7 +22,8 @@ from ul_api_utils.conf import APPLICATION_ENV, APPLICATION_JWT_PUBLIC_KEY, APPLI
 from ul_api_utils.const import REQUEST_HEADER__X_FORWARDED_FOR, \
     RESPONSE_HEADER__WWW_AUTH, OOPS, REQUEST_HEADER__USER_AGENT
 from ul_api_utils.errors import ValidationListApiError, AccessApiError, NoResultFoundApiError, PermissionDeniedApiError, \
-    SimpleValidateApiError, ValidateApiError, HasAlreadyExistsApiError, AbstractInternalApiError, InvalidContentTypeError
+    SimpleValidateApiError, ValidateApiError, HasAlreadyExistsApiError, AbstractInternalApiError, \
+    InvalidContentTypeError
 from ul_api_utils.internal_api.internal_api_response import InternalApiResponse
 from ul_api_utils.modules.api_sdk_config import ApiSdkConfig
 from ul_api_utils.modules.api_sdk_jwt import ApiSdkJwt, JWT_VERSION
@@ -88,6 +91,9 @@ class ApiResource:
         self._limiter_enabled = limiter_enabled
         self._db_initialized = db_initialized
 
+    def __repr__(self) -> str:
+        return f"ApiResource object. Type: {self._type}, Function: {self._fn_typing.fn.__name__}"
+
     @property
     def debugger_enabled(self) -> bool:
         return self._debugger_enabled
@@ -208,30 +214,51 @@ class ApiResource:
 
         return JsonApiResponse._internal_use_response_error(self._fn_typing.response_payload_many, 500, [self._mk_error("system-error", str(err))])
 
+    def _generate_web_error_response(
+        self,
+        happened_exception: Exception,
+        default_status_code: int,
+        error_message: str,
+        apply_auth_headers: bool = False,
+    ) -> HtmlApiResponse:
+        if hasattr(happened_exception, "status_code"):
+            default_status_code = happened_exception.status_code  # type: ignore
+        assert self._config.web_error_template is not None  # only for mypy
+        response = HtmlApiResponse(
+            content=self.render_template(
+                self._config.web_error_template,
+                {
+                    "error_traceback": happened_exception,
+                    "error_message": error_message,
+                    "status_code": default_status_code,
+                },
+            ),
+            ok=False,
+            status_code=default_status_code,
+        )
+        if apply_auth_headers:
+            if self.method != ApiMethod.OPTIONS and self._config.http_auth is not None:
+                response.headers[RESPONSE_HEADER__WWW_AUTH] = f'{self._config.http_auth.scheme} realm="{self._config.http_auth.realm}"'
+            return response
+        return response
+
     def response_web_error(self, err: Exception) -> HtmlApiResponse:
-        if isinstance(err, PermissionDeniedApiError):
-            resp = HtmlApiResponse(error=err, content=f"403 Access Not Permitted. {str(err)}", ok=False, status_code=403)
-            if self.method != ApiMethod.OPTIONS and self._config.http_auth is not None:
-                resp.headers[RESPONSE_HEADER__WWW_AUTH] = f'{self._config.http_auth.scheme} realm="{self._config.http_auth.realm}"'
-            return resp
-        if isinstance(err, AccessApiError):
-            resp = HtmlApiResponse(error=err, content=f"401 Invalid Token. {str(err)}", ok=False, status_code=401)
-            if self.method != ApiMethod.OPTIONS and self._config.http_auth is not None:
-                resp.headers[RESPONSE_HEADER__WWW_AUTH] = f'{self._config.http_auth.scheme} realm="{self._config.http_auth.realm}"'
-            return resp
-        if isinstance(err, ValidateApiError):
-            return HtmlApiResponse(error=err, content="400 Invalid Request\n Request validation error", ok=False, status_code=400)
+        if self._config.web_error_template is None:
+            return HtmlApiResponse(error=err, content=OOPS, ok=False, status_code=500)
 
         if self._db_initialized:
             from sqlalchemy.exc import NoResultFound as NoResultFoundError
 
-            if isinstance(err, NoResultFoundError):
-                if self._config.not_found_template is None:
-                    return HtmlApiResponse(error=err, content="404 Not Found", ok=False, status_code=404)
-                return HtmlApiResponse(error=err, content=self.render_template(self._config.not_found_template, {}), ok=False, status_code=404)
-        if self._config.not_found_template is None:
-            return HtmlApiResponse(error=err, content=OOPS, ok=False, status_code=500)
-        return HtmlApiResponse(error=err, content=OOPS, ok=False, status_code=500)
+            WEB_EXCEPTION_HANDLING_PARAMS__MAP[NoResultFoundError] = ProcessingExceptionsParams(default_status_code=404, error_message="Not found")
+
+        error_response_params = WEB_EXCEPTION_HANDLING_PARAMS__MAP.get(type(err), WEB_UNKNOWN_ERROR_PARAMS)
+
+        return self._generate_web_error_response(
+            happened_exception=err,
+            default_status_code=error_response_params.default_status_code,
+            error_message=error_response_params.error_message,
+            apply_auth_headers=error_response_params.apply_auth_headers,
+        )
 
     def render_template(self, template_name: str, data: Dict[str, Any]) -> str:
         return render_template(
