@@ -8,12 +8,12 @@ from base64 import b64decode
 from datetime import datetime
 from functools import wraps
 from typing import List, Union, Callable, Tuple, Any, Optional, TypeVar, cast, Set, TYPE_CHECKING
-
+from ul_api_utils.resources.sockets import socket_io
 from flask import Response, request, Flask, url_for as flask_url_for, _app_ctx_stack
 from pydantic import BaseModel
+from redis.connection import parse_url
 from ul_py_tool.utils.arg_files_glob import arg_files_print
 from werkzeug import Response as BaseResponse
-
 from ul_api_utils.access import PermissionDefinition, GLOBAL_PERMISSION__PRIVATE, GLOBAL_PERMISSION__PRIVATE_RT, \
     GLOBAL_PERMISSION__PUBLIC, PermissionRegistry
 from ul_api_utils.api_resource.api_resource import ApiResource
@@ -21,15 +21,15 @@ from ul_api_utils.api_resource.api_resource_config import ApiResourceConfig
 from ul_api_utils.api_resource.api_resource_fn_typing import ApiResourceFnTyping
 from ul_api_utils.api_resource.api_resource_type import ApiResourceType
 from ul_api_utils.api_resource.api_response import ApiResponse, JsonApiResponse, RootJsonApiResponse, ProxyJsonApiResponse, AnyJsonApiResponse
-from ul_api_utils.conf import APPLICATION_DEBUGGER_PIN, APPLICATION_DEBUG, APPLICATION_DIR, APPLICATION_ENV_IS_LOCAL, \
-    APPLICATION_START_DT
+from ul_api_utils.conf import APPLICATION_DEBUGGER_PIN, APPLICATION_DEBUG, APPLICATION_DIR, APPLICATION_ENV_IS_LOCAL, APPLICATION_START_DT, APPLICATION_DEBUG_LOGGING
 from ul_api_utils.const import REQUEST_HEADER__DEBUGGER, REQUEST_HEADER__INTERNAL
 from ul_api_utils.debug import stat
 from ul_api_utils.debug.debugger import Debugger
 from ul_api_utils.errors import UserAbstractApiError, ResourceRuntimeApiError, ResponseTypeRuntimeApiError
 from ul_api_utils.internal_api.internal_api_check_context import internal_api_check_context
-from ul_api_utils.modules.api_sdk_config import ApiSdkConfig
+from ul_api_utils.modules.api_sdk_config import ApiSdkConfig, SocketAsyncModesEnum
 from ul_api_utils.modules.intermediate_state import try_init, try_configure
+from ul_api_utils.resources.caching import ULCache, TCacheMode, ULCacheMode, ULCacheConfig
 from ul_api_utils.resources.debugger_scripts import load_debugger_static_scripts
 from ul_api_utils.resources.health_check.health_check import HealthCheckContext
 from ul_api_utils.resources.health_check.resource import init_health_check_resource
@@ -51,7 +51,9 @@ from ul_api_utils.utils.uuid_converter import UUID4Converter
 
 if TYPE_CHECKING:
     import flask_sqlalchemy
-    from ul_db_utils.modules.db import DbConfig
+    from flask_mongoengine import MongoEngine      # type: ignore # lib without mypy stubs
+    from ul_db_utils.modules.postgres_modules.db import DbConfig
+    from ul_db_utils.modules.mongo_db_modules.db import MongoDbConfig
 
 TFn = TypeVar("TFn", bound=Callable[..., ApiResponse])
 
@@ -104,6 +106,7 @@ class ApiSdk:
         '_fn_registry',
         '_flask_app_cache',
         '_limiter_enabled',
+        '_cache',
         '_db',
     )
 
@@ -116,22 +119,30 @@ class ApiSdk:
         self._initialized_flask_name: Optional[str] = None
         self._flask_app_cache: Optional[Flask] = None
         self._limiter_enabled = False
+        self._cache = None
 
         self._templates_dir = os.path.join(APPLICATION_DIR, 'templates')
 
         self._fn_registry: List[ApiSdkResource] = []
-        self._db: Optional['flask_sqlalchemy.SQLAlchemy'] = None
+        self._db: Optional['flask_sqlalchemy.SQLAlchemy'] | Optional['MongoEngine'] = None
 
     @property
     def config(self) -> ApiSdkConfig:
         return self._config
 
-    def init_with_flask(self, app_name: str, *, db_config: Optional['DbConfig'] = None) -> Flask:
+    def init_with_flask(self, app_name: str, *, db_config: Optional['MongoDbConfig'] | Optional['DbConfig'] = None) -> Flask:
         self._initialized_flask_name = try_init(self, app_name)
 
-        if db_config is not None:
+        if db_config is not None and type(db_config).__name__ == 'MongoDbConfig':
+            from ul_db_utils.utils.waiting_for_mongo import waiting_for_mongo
+            from ul_db_utils.modules.mongo_db_modules.db import db
+            db_config._init_from_sdk_with_flask(self)
+            waiting_for_mongo(db_config.uri)
+            self._db = db
+
+        if db_config is not None and type(db_config).__name__ == 'DbConfig':
             from ul_db_utils.utils.waiting_for_postgres import waiting_for_postgres
-            from ul_db_utils.modules.db import db
+            from ul_db_utils.modules.postgres_modules.db import db       # type: ignore # yes, db already defined, but can not use two conigs
             db_config._init_from_sdk_with_flask(self)
             waiting_for_postgres(db_config.uri)
             self._db = db
@@ -144,6 +155,26 @@ class ApiSdk:
             rate_limit=self._config.rate_limit,
             storage_uri=self._config.rate_limit_storage_uri,
         )
+        if self._config.cache_storage_uri:
+            cache_config: ULCacheConfig = {
+                'CACHE_TYPE': "RedisCache",
+                'CACHE_REDIS_HOST': '',
+                'CACHE_REDIS_PORT': '',
+                'CACHE_REDIS_PASSWORD': '',
+                'CACHE_DEFAULT_TIMEOUT': self._config.cache_default_ttl,
+                'CACHE_KEY_PREFIX': f'CACHE__{self._config.service_name}',
+                'CACHE_SOURCE_CHECK': True,
+            }
+            try:
+                redis_url = parse_url(self._config.cache_storage_uri)
+            except ValueError as e:
+                logger.error(f'broken redis uri :: {e}')
+            else:
+                assert all(('host' in redis_url, 'port' in redis_url)), 'missing part of redis uri'
+                cache_config['CACHE_REDIS_HOST'] = redis_url['host']
+                cache_config['CACHE_REDIS_PORT'] = redis_url['port']
+                cache_config['CACHE_REDIS_PASSWORD'] = redis_url.get('password', '')
+            self._cache = ULCache(self._flask_app, config=cache_config)     # type: ignore
 
         route_files, ignored_route_files = load_modules_by_template([
             os.path.join(APPLICATION_DIR, 'routes', 'api_*.py'),
@@ -155,12 +186,28 @@ class ApiSdk:
         if APPLICATION_DEBUG:
             arg_files_print(1000, route_files, ignored_files=ignored_route_files, name='files loaded')
 
+            if (plugins_config := self.config.flask_debugging_plugins) is not None:
+                if plugins_config.flask_monitoring_dashboard:
+                    import flask_monitoringdashboard as dashboard  # type: ignore
+                    if os.environ.get('FLASK_MONITORING_DASHBOARD_CONFIG'):
+                        dashboard.config.init_from(envvar='FLASK_MONITORING_DASHBOARD_CONFIG', log_verbose=True)
+                    dashboard.bind(self._flask_app)
+
         load_permissions(self, self._initialized_flask_name, self._config.permissions)
 
         load_debugger_static_scripts(self)
 
         load_swagger(self, self._fn_registry, self._config.api_route_path_prefix)
 
+        if self.config.sockets_config:
+            socket_io.init_app(
+                self._flask_app,
+                message_queue=self.config.sockets_config.message_queue,
+                async_mode=SocketAsyncModesEnum.GEVENT.value,
+                channel=self.config.sockets_config.channel,
+                cors_allowed_origins=self.config.sockets_config.cors_allowed_origins,
+            )
+
         return self._flask_app
 
     @property
@@ -187,12 +234,11 @@ class ApiSdk:
         flask_app.config['EXPLAIN_TEMPLATE_LOADING'] = False
         flask_app.config['ENV'] = 'development' if APPLICATION_DEBUG else 'production'
         flask_app.config['SECRET_KEY'] = 'some-long-long-secret-only-for-wtforms-string-be-brave-if-you-use-it-on-prod'
-
         flask_app.config['TEMPLATES_AUTO_RELOAD'] = APPLICATION_DEBUG and APPLICATION_ENV_IS_LOCAL
         flask_app.config['JSONIFY_PRETTYPRINT_REGULAR'] = False
         flask_app.config['JSON_SORT_KEYS'] = False
         flask_app.config['PREFERRED_URL_SCHEME'] = 'http'
-        flask_app.config['APPLICATION_ROOT'] = APPLICATION_DIR
+        flask_app.config['APPLICATION_ROOT'] = '/'
 
         flask_app.add_template_filter(to_pretty_json, 'tojson_pretty')
         flask_app.add_template_filter(to_pretty_json, 'to_json_pretty')
@@ -201,7 +247,7 @@ class ApiSdk:
 
         flask_app.before_request(self._before_request)
         flask_app.after_request(self._after_request)
-        flask_app.teardown_request(self._teardown)
+        flask_app.teardown_request(self._teardown_request)
 
         flask_app.errorhandler(404)(functools.partial(
             not_implemented_handler,
@@ -212,7 +258,11 @@ class ApiSdk:
         return flask_app
 
     @property
-    def db(self) -> Optional['flask_sqlalchemy.SQLAlchemy']:
+    def ul_cache_factory(self) -> Optional['ULCache']:
+        return self._cache
+
+    @property
+    def db(self) -> Optional['flask_sqlalchemy.SQLAlchemy'] | Optional['MongoEngine']:
         return self._db
 
     def _before_request(self) -> None:
@@ -224,15 +274,16 @@ class ApiSdk:
 
     def _after_request(self, response: Response) -> Response:
         assert self._initialized_flask_name is not None
-        if not self._debugger_enabled_with_pin() and APPLICATION_ENV_IS_LOCAL and self._debug_internal_request():
+        if APPLICATION_DEBUG and (APPLICATION_DEBUG_LOGGING or APPLICATION_ENV_IS_LOCAL):
             d = Debugger(self._initialized_flask_name, True, ApiMethod(request.method), request.url)
             d.render_console()
+            # if not self._debugger_enabled_with_pin() and APPLICATION_ENV_IS_LOCAL and self._debug_internal_request():
         return response
 
     def _debugger_enabled_with_pin(self) -> bool:
         return request.headers.get(REQUEST_HEADER__DEBUGGER) == APPLICATION_DEBUGGER_PIN or request.cookies.get(REQUEST_HEADER__DEBUGGER, "") == APPLICATION_DEBUGGER_PIN
 
-    def _teardown(self, err: Optional[BaseException]) -> None:
+    def _teardown_request(self, err: Optional[BaseException]) -> None:
         clean_files()
 
     def _debug_internal_request(self) -> bool:
@@ -270,6 +321,20 @@ class ApiSdk:
     def health_check(self) -> Callable[[Callable[[HealthCheckContext], None]], None]:
         return functools.partial(init_health_check_resource, api_sdk=self)
 
+    def cache_api(
+        self,
+        mode: TCacheMode,
+        tags: Tuple[str, ...] | str,
+        timeout: Optional[int] = None,
+        source_check: Optional[bool] = True,
+    ) -> Callable[[TFn], TFn]:
+        if self.ul_cache_factory is None:
+            logger.warning("Cache URI is not configured, cache will not work.")
+            return lambda fn: fn
+        if ULCacheMode.compile_mode(mode) == ULCacheMode.READ.value:
+            return self.ul_cache_factory.cache_read_wrap(tags, timeout, source_check)
+        return self.ul_cache_factory.cache_refresh_wrap(tags)
+
     def rest_api(
         self,
         method: TMethodShort,
@@ -421,6 +486,7 @@ class ApiSdk:
                     res = api_resource_config.override_flask_response(res)
 
                 ri = api_resource.request_info
+                # TODO: add cache hit flag
                 logger.info('AUDIT ' + json.dumps({
                     'user_id': str(scope_user_id) if scope_user_id else None,
                     'token_id': str(scope_token_id) if scope_token_id else None,
